@@ -25,27 +25,62 @@
 #include <string.h>
 #include <assert.h>
 
+#include "modbus.h"
+
+#if defined(HAVE_BYTESWAP_H)
+#  include <byteswap.h>
+#endif
+
+#if defined(__GNUC__)
+#  define GCC_VERSION (__GNUC__ * 100 + __GNUC_MINOR__ * 10)
+#  if GCC_VERSION >= 430
+// Since GCC >= 4.30, GCC provides __builtin_bswapXX() alternatives so we switch to them
+#    undef bswap_32
+#    define bswap_32 __builtin_bswap32
+#  endif
+#endif
+#if defined(_MSC_VER) && (_MSC_VER >= 1400)
+# define bswap_32 _byteswap_ulong
+#endif
+
+#if !defined(bswap_32)
+
+#if !defined(bswap_16)
+#   warning "Fallback on C functions for bswap_16"
+static inline uint16_t bswap_16(uint16_t x)
+{
+    return (x >> 8) | (x << 8);
+}
+#endif
+
+#   warning "Fallback on C functions for bswap_32"
+static inline uint32_t bswap_32(uint32_t x)
+{
+    return (bswap_16(x & 0xffff) << 16) | (bswap_16(x >> 16));
+}
+#endif
+
 /* Sets many bits from a single byte value (all 8 bits of the byte value are
    set) */
-void modbus_set_bits_from_byte(uint8_t *dest, int index, const uint8_t value)
+void modbus_set_bits_from_byte(uint8_t *dest, int idx, const uint8_t value)
 {
     int i;
 
-    for (i=0; i<8; i++) {
-        dest[index+i] = (value & (1 << i)) ? 1 : 0;
+    for (i=0; i < 8; i++) {
+        dest[idx+i] = (value & (1 << i)) ? 1 : 0;
     }
 }
 
-/* Sets many bits from a table of bytes (only the bits between index and
-   index + nb_bits are set) */
-void modbus_set_bits_from_bytes(uint8_t *dest, int index, unsigned int nb_bits,
+/* Sets many bits from a table of bytes (only the bits between idx and
+   idx + nb_bits are set) */
+void modbus_set_bits_from_bytes(uint8_t *dest, int idx, unsigned int nb_bits,
                                 const uint8_t *tab_byte)
 {
-    int i;
+    unsigned int i;
     int shift = 0;
 
-    for (i = index; i < index + nb_bits; i++) {
-        dest[i] = tab_byte[(i - index) / 8] & (1 << shift) ? 1 : 0;
+    for (i = idx; i < idx + nb_bits; i++) {
+        dest[i] = tab_byte[(i - idx) / 8] & (1 << shift) ? 1 : 0;
         /* gcc doesn't like: shift = (++shift) % 8; */
         shift++;
         shift %= 8;
@@ -54,10 +89,10 @@ void modbus_set_bits_from_bytes(uint8_t *dest, int index, unsigned int nb_bits,
 
 /* Gets the byte value from many bits.
    To obtain a full byte, set nb_bits to 8. */
-uint8_t modbus_get_byte_from_bits(const uint8_t *src, int index,
+uint8_t modbus_get_byte_from_bits(const uint8_t *src, int idx,
                                   unsigned int nb_bits)
 {
-    int i;
+    unsigned int i;
     uint8_t value = 0;
 
     if (nb_bits > 8) {
@@ -67,16 +102,16 @@ uint8_t modbus_get_byte_from_bits(const uint8_t *src, int index,
     }
 
     for (i=0; i < nb_bits; i++) {
-        value |= (src[index+i] << i);
+        value |= (src[idx+i] << i);
     }
 
     return value;
 }
 
-/* Get a float from 4 bytes in Modbus format */
+/* Get a float from 4 bytes in Modbus format (ABCD) */
 float modbus_get_float(const uint16_t *src)
 {
-    float f = 0.0f;
+    float f;
     uint32_t i;
 
     i = (((uint32_t)src[1]) << 16) + src[0];
@@ -85,12 +120,35 @@ float modbus_get_float(const uint16_t *src)
     return f;
 }
 
-/* Set a float to 4 bytes in Modbus format */
+/* Get a float from 4 bytes in inversed Modbus format (DCBA) */
+float modbus_get_float_dcba(const uint16_t *src)
+{
+    float f;
+    uint32_t i;
+
+    i = bswap_32((((uint32_t)src[1]) << 16) + src[0]);
+    memcpy(&f, &i, sizeof(float));
+
+    return f;
+}
+
+/* Set a float to 4 bytes in Modbus format (ABCD) */
 void modbus_set_float(float f, uint16_t *dest)
 {
-    uint32_t i = 0;
+    uint32_t i;
 
     memcpy(&i, &f, sizeof(uint32_t));
     dest[0] = (uint16_t)i;
     dest[1] = (uint16_t)(i >> 16);
 }
+
+/* Set a float to 4 bytes in inversed Modbus format (DCBA) */
+void modbus_set_float_dcba(float f, uint16_t *dest)
+{
+    uint32_t i;
+
+    memcpy(&i, &f, sizeof(uint32_t));
+    i = bswap_32(i);
+    dest[0] = (uint16_t)i;
+    dest[1] = (uint16_t)(i >> 16);
+}
