@@ -16,8 +16,8 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#ifndef _MODBUS_RTU_PRIVATE_H_
-#define _MODBUS_RTU_PRIVATE_H_
+#ifndef MODBUS_RTU_PRIVATE_H
+#define MODBUS_RTU_PRIVATE_H
 
 #ifndef _MSC_VER
 #include <stdint.h>
@@ -37,32 +37,30 @@
 
 #define _MODBUS_RTU_CHECKSUM_LENGTH    2
 
+/* Time waited beetween the RTS switch before transmit data or after transmit
+   data before to read */
+#define _MODBUS_RTU_TIME_BETWEEN_RTS_SWITCH 10000
+
 #if defined(_WIN32)
+#if !defined(ENOTSUP)
 #define ENOTSUP WSAEOPNOTSUPP
+#endif
 
 /* WIN32: struct containing serial handle and a receive buffer */
 #define PY_BUF_SIZE 512
 struct win32_ser {
-	/* File handle */
-	HANDLE fd;
-	/* Receive buffer */
-	uint8_t buf[PY_BUF_SIZE];
-	/* Received chars */
-	DWORD n_bytes;
+    /* File handle */
+    HANDLE fd;
+    /* Receive buffer */
+    uint8_t buf[PY_BUF_SIZE];
+    /* Received chars */
+    DWORD n_bytes;
 };
 #endif /* _WIN32 */
 
 typedef struct _modbus_rtu {
-    /* Device: "/dev/ttyS0", "/dev/ttyUSB0" or "/dev/tty.USA19*" on Mac OS X for
-       KeySpan USB<->Serial adapters this string had to be made bigger on OS X
-       as the directory+file name was bigger than 19 bytes. Making it 67 bytes
-       for now, but OS X does support 256 byte file names. May become a problem
-       in the future. */
-#if defined(__APPLE_CC__)
-    char device[64];
-#else
-    char device[16];
-#endif
+    /* Device: "/dev/ttyS0", "/dev/ttyUSB0" or "/dev/tty.USA19*" on Mac OS X. */
+    char *device;
     /* Bauds: 9600, 19200, 57600, 115200, etc */
     int baud;
     /* Data bit */
@@ -81,6 +79,12 @@ typedef struct _modbus_rtu {
 #if HAVE_DECL_TIOCSRS485
     int serial_mode;
 #endif
+#if HAVE_DECL_TIOCM_RTS
+    int rts;
+    int onebyte_time;
+#endif
+    /* To handle many slaves on the same link */
+    int confirmation_to_ignore;
 } modbus_rtu_t;
 
-#endif /* _MODBUS_RTU_PRIVATE_H_ */
+#endif /* MODBUS_RTU_PRIVATE_H */
