@@ -16,8 +16,8 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#ifndef _MODBUS_RTU_H_
-#define _MODBUS_RTU_H_
+#ifndef MODBUS_RTU_H
+#define MODBUS_RTU_H
 
 #include "modbus.h"
 
@@ -28,15 +28,22 @@ MODBUS_BEGIN_DECLS
  */
 #define MODBUS_RTU_MAX_ADU_LENGTH  256
 
-modbus_t* modbus_new_rtu(const char *device, int baud, char parity,
-                         int data_bit, int stop_bit);
+MODBUS_API modbus_t* modbus_new_rtu(const char *device, int baud, char parity,
+                                int data_bit, int stop_bit);
 
 #define MODBUS_RTU_RS232 0
 #define MODBUS_RTU_RS485 1
 
-int modbus_rtu_set_serial_mode(modbus_t *ctx, int mode);
-int modbus_rtu_get_serial_mode(modbus_t *ctx);
+MODBUS_API int modbus_rtu_set_serial_mode(modbus_t *ctx, int mode);
+MODBUS_API int modbus_rtu_get_serial_mode(modbus_t *ctx);
+
+#define MODBUS_RTU_RTS_NONE   0
+#define MODBUS_RTU_RTS_UP     1
+#define MODBUS_RTU_RTS_DOWN   2
+
+MODBUS_API int modbus_rtu_set_rts(modbus_t *ctx, int mode);
+MODBUS_API int modbus_rtu_get_rts(modbus_t *ctx);
 
 MODBUS_END_DECLS
 
-#endif /* _MODBUS_RTU_H_ */
+#endif /* MODBUS_RTU_H */
