@@ -1,22 +1,14 @@
 /*
- * Copyright © 2008-2010 Stéphane Raimbault <stephane.raimbault@gmail.com>
+ * Copyright © 2008-2014 Stéphane Raimbault <stephane.raimbault@gmail.com>
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * it under the terms of the BSD License.
  */
 
 #include <stdio.h>
+#ifndef _MSC_VER
 #include <unistd.h>
+#endif
 #include <stdlib.h>
 #include <errno.h>
 
@@ -24,7 +16,7 @@
 
 int main(void)
 {
-    int socket;
+    int s = -1;
     modbus_t *ctx;
     modbus_mapping_t *mb_mapping;
 
@@ -39,18 +31,18 @@ int main(void)
         return -1;
     }
 
-    socket = modbus_tcp_listen(ctx, 1);
-    modbus_tcp_accept(ctx, &socket);
+    s = modbus_tcp_listen(ctx, 1);
+    modbus_tcp_accept(ctx, &s);
 
     for (;;) {
         uint8_t query[MODBUS_TCP_MAX_ADU_LENGTH];
         int rc;
 
         rc = modbus_receive(ctx, query);
-        if (rc != -1) {
+        if (rc > 0) {
             /* rc is the query size */
             modbus_reply(ctx, query, rc, mb_mapping);
-        } else {
+        } else if (rc == -1) {
             /* Connection closed by the client or error */
             break;
         }
@@ -58,6 +50,9 @@ int main(void)
 
     printf("Quit the loop: %s\n", modbus_strerror(errno));
 
+    if (s != -1) {
+        close(s);
+    }
     modbus_mapping_free(mb_mapping);
     modbus_close(ctx);
     modbus_free(ctx);
